using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.IO;

namespace FileFinder
{
    public sealed class FindFilesCompletedEventArgs : EventArgs
    {
        private readonly FileInfo[] results;

        public IEnumerable<FileInfo> Results
        {
            get { return results; }
        }

        public FindFilesCompletedEventArgs(List<FileInfo> operationResults)
        {
            this.results = operationResults.ToArray();
        }
    }

    public sealed class FindFilesProgressEventArgs : EventArgs
    {
        private readonly FileInfo currentFile;

        public FileInfo CurrentFile
        {
            get { return currentFile; }
        } 
        
        public FindFilesProgressEventArgs(FileInfo file)
        {
            currentFile = file;
        }
    }

    public class FindFilesWorker
    {
        public event EventHandler<FindFilesCompletedEventArgs> OnEndWork;
        public event EventHandler<FindFilesProgressEventArgs> OnWorkProgress;

        private static BackgroundWorker engine;

        private readonly string rootDir;
        private readonly TimeSpan epoch;

        private List<FileInfo> fileStats = new List<FileInfo>();

        private BackgroundWorker currentBGWorker;

        public FindFilesWorker(string root, TimeSpan start)
        {
            rootDir = root;
            epoch = start;

            if (engine == null)
            {
                engine = new BackgroundWorker();
                engine.DoWork += new DoWorkEventHandler(engine_DoWork);
                engine.RunWorkerCompleted += new RunWorkerCompletedEventHandler(engine_RunWorkerCompleted);
                engine.WorkerReportsProgress = true;
                engine.ProgressChanged += new ProgressChangedEventHandler(engine_ProgressChanged);
                engine.WorkerSupportsCancellation = true;
            }
        }

        void engine_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            FileInfo info = e.UserState as FileInfo;
            if (OnWorkProgress != null)
                OnWorkProgress(this, new FindFilesProgressEventArgs(info));
            fileStats.Add(info);
            
        }

        public void DoWork()
        {
            IEnumerable<FileInfo> themFiles = FileUtilities.FilesChanged(rootDir, epoch);
            if (currentBGWorker != null)
                foreach (FileInfo file in themFiles)
                {
                    currentBGWorker.ReportProgress(0, file);
                    if (currentBGWorker.CancellationPending)
                        return;
                }
            else
                fileStats.AddRange(themFiles);
        }

        void engine_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (OnEndWork != null)
            {
                OnEndWork(this, new FindFilesCompletedEventArgs(fileStats));
            }
            currentBGWorker = null;
        }

        public bool StartWork()
        {
            if (!engine.IsBusy)
            {
                engine.RunWorkerAsync(this);
                return true;
            }
            else
                return false;
        }

        public void RequestCancel()
        {
            if (currentBGWorker != null)
                currentBGWorker.CancelAsync();
        }

        private static void engine_DoWork(object sender, DoWorkEventArgs e)
        {
            FindFilesWorker args = e.Argument as FindFilesWorker;
            args.currentBGWorker = sender as BackgroundWorker;
            args.DoWork();
        }

    }
}
