Option Strict On
Option Explicit On

Imports System
Imports System.Collections.Generic

Public MustInherit Class BindingSupportDecoratorBase
   Inherits Windows.Controls.Decorator

   Private mDataBindings As List(Of DataBindingInfo)

   Protected Delegate Sub ActionDelegate( _
            ByVal frameworkElement As Windows.FrameworkElement, _
            ByVal binding As Windows.Data.Binding, _
            ByVal prop As Windows.DependencyProperty, _
            ByVal associatedLabel As Windows.Controls.Label)

   Public Sub New()
      AddHandler Me.DataContextChanged, AddressOf Decorator_DataContextChanged
   End Sub

   Private ReadOnly Property DataBindings() As List(Of DataBindingInfo)
      Get
         If mDataBindings Is Nothing Then
            Refresh()
         End If
         Return mDataBindings
      End Get
   End Property

   Public Overridable Sub Refresh()
      mDataBindings = New List(Of DataBindingInfo)
      RefreshBindings(Me)
   End Sub

   Private Sub Decorator_DataContextChanged( _
            ByVal sender As Object, _
            ByVal e As Windows.DependencyPropertyChangedEventArgs)
      Refresh()
   End Sub

   Protected Overridable Sub Action( _
            ByVal frameworkElement As Windows.FrameworkElement, _
            ByVal binding As Windows.Data.Binding, _
            ByVal prop As Windows.DependencyProperty, _
            ByVal associatedLabel As Windows.Controls.Label)
      ' Derived classes can select their own actions
   End Sub

   Protected Sub ForAll()
      For Each dataBindingInfo As DataBindingInfo In Me.DataBindings
         Action(dataBindingInfo.FrameworkElement, dataBindingInfo.Binding, _
                  dataBindingInfo.DependencyProperty, dataBindingInfo.AssociatedLabel)
      Next
   End Sub

   Protected Sub ForAll(ByVal action As ActionDelegate)
      For Each dataBindingInfo As DataBindingInfo In Me.DataBindings
         action.Invoke(dataBindingInfo.FrameworkElement, dataBindingInfo.Binding, dataBindingInfo.DependencyProperty, dataBindingInfo.AssociatedLabel)
      Next
   End Sub

   Private Sub RefreshBindings( _
               ByVal element As Windows.DependencyObject)
      Dim sharedMembers As Reflection.MemberInfo() = element.GetType().GetMembers(Reflection.BindingFlags.Static Or Reflection.BindingFlags.Public Or Reflection.BindingFlags.FlattenHierarchy)
      Dim dependencyObject As Windows.DependencyObject = TryCast(element, Windows.DependencyObject)
      For Each member As Reflection.MemberInfo In sharedMembers
         'If member.Name.StartsWith("Text") Then
         '   Console.WriteLine("Fred")
         'End If
         Dim prop As Windows.DependencyProperty = Nothing
         If member.MemberType = Reflection.MemberTypes.Field Then
            prop = TryCast((CType(member, Reflection.FieldInfo)).GetValue(Child), Windows.DependencyProperty)
         ElseIf member.MemberType = Reflection.MemberTypes.Property Then
            prop = TryCast((CType(member, Reflection.PropertyInfo)).GetValue(Child, Nothing), Windows.DependencyProperty)
            Dim test As Windows.Controls.TextBox = TryCast(element, Windows.Controls.TextBox)
         End If

         If Not prop Is Nothing Then
            Dim binding As Windows.Data.Binding = Windows.Data.BindingOperations.GetBinding(dependencyObject, prop)
            Dim frameworkElement As Windows.FrameworkElement = TryCast(element, Windows.FrameworkElement)
            If binding IsNot Nothing AndAlso _
                     frameworkElement IsNot Nothing AndAlso _
                     binding.RelativeSource Is Nothing AndAlso _
                     binding.Path IsNot Nothing AndAlso _
                     String.IsNullOrEmpty(binding.ElementName) Then
               Dim associatedLabel As Windows.Controls.Label = GetAssociatedLabel(Me, frameworkElement)
               DataBindings.Add(New DataBindingInfo(frameworkElement, binding, prop, associatedLabel))
            End If
         End If
      Next
      For Each child As Object In Windows.LogicalTreeHelper.GetChildren(element)
         Dim childDependencyObject As Windows.DependencyObject = TryCast(child, Windows.DependencyObject)
         If childDependencyObject IsNot Nothing Then
            RefreshBindings(childDependencyObject)
         End If
      Next
   End Sub

   ''' <summary>
   ''' Gets the first label associated with the element. Multiple
   ''' elements are possible but this case seems so bizarre I am 
   ''' ignoring it
   ''' </summary>
   Private Function GetAssociatedLabel( _
            ByVal searchIn As Windows.DependencyObject, _
            ByVal frameworkElement As Windows.FrameworkElement) _
            As Windows.Controls.Label
      Dim label As Windows.Controls.Label = Nothing
      ' Can't target something without a name
      If Not String.IsNullOrEmpty(frameworkElement.Name) Then
         label = TryCast(searchIn, Windows.Controls.Label)
         If label IsNot Nothing Then
            Dim target As Windows.UIElement = label.Target
            If target Is frameworkElement Then
               Return label
            End If
         End If
         For Each child As Object In Windows.LogicalTreeHelper.GetChildren(searchIn)
            Dim childDependencyObject As Windows.DependencyObject = TryCast(child, Windows.DependencyObject)
            If childDependencyObject IsNot Nothing Then
               label = GetAssociatedLabel(childDependencyObject, frameworkElement)
               If label IsNot Nothing Then
                  Return label
               End If
            End If
         Next
      End If
      Return Nothing
   End Function

   Protected ReadOnly Property BoundObject() As BObjects.BizObject
      Get
         Dim dataSource As Object = Nothing
         Dim dataSourceProvider As Windows.Data.DataSourceProvider = TryCast(Me.DataContext, Windows.Data.DataSourceProvider)
         If dataSourceProvider IsNot Nothing Then
            dataSource = dataSourceProvider.Data
         Else
            dataSource = Me.DataContext
         End If
         Dim bizObject As BObjects.BizObject = TryCast(dataSource, BObjects.BizObject)
         If bizObject Is Nothing Then
            Dim collection As ComponentModel.ICollectionView = Windows.Data.CollectionViewSource.GetDefaultView(Me.DataContext)
            bizObject = TryCast(collection.CurrentItem, BObjects.BizObject)
         End If
         Return bizObject
      End Get
   End Property

   Private Class DataBindingInfo
      Private mBinding As Windows.Data.Binding
      Private mFrameworkElement As Windows.FrameworkElement
      Private mDependencyProperty As Windows.DependencyProperty
      Private mAssociatedLabel As Windows.Controls.Label

      Public Sub New( _
               ByVal frameworkelement As Windows.FrameworkElement, _
               ByVal binding As Windows.Data.Binding, _
               ByVal dependencyProperty As Windows.DependencyProperty, _
               ByVal associatedLabel As Windows.Controls.Label)
         Me.mBinding = binding
         Me.mFrameworkElement = frameworkelement
         Me.mDependencyProperty = dependencyProperty
         Me.mAssociatedLabel = associatedLabel
      End Sub


      Public ReadOnly Property Binding() As Windows.Data.Binding
         Get
            Return Me.mBinding
         End Get
      End Property

      Public ReadOnly Property FrameworkElement() As Windows.FrameworkElement
         Get
            Return Me.mFrameworkElement
         End Get
      End Property

      Public ReadOnly Property DependencyProperty() As Windows.DependencyProperty
         Get
            Return Me.mDependencyProperty
         End Get
      End Property

      Public ReadOnly Property AssociatedLabel() As Windows.Controls.Label
         Get
            Return Me.mAssociatedLabel
         End Get
      End Property

   End Class

End Class
