''' <summary>
''' This attribute should be applied to the fields or properties of a 
''' row object to indicate that the member receives the value of
''' a particular column.
''' </summary>
''' <remarks>
''' <para>If the <see cref="ColumnAttribute.ColumnName"/> property is null 
''' it is assumed to be the same as the attributed member's name.</para>
''' </remarks>
<AttributeUsage(AttributeTargets.Field Or AttributeTargets.Property, AllowMultiple:=False, Inherited:=True)> _
Public Class ColumnAttribute
    Inherits DbAttribute

    Private _ColumnName As String
    Private _NullValue As Object

    ''' <summary>
    ''' Constructs a new instance of the attribute.
    ''' </summary>
    Public Sub New()
    End Sub

    ''' <summary>
    ''' Constructs a new instance of the attribute, tagging the attributed member
    ''' with the specified column name.
    ''' </summary>
    Public Sub New(ByVal ColumnName As String)
        _ColumnName = ColumnName
    End Sub

    ''' <summary>
    ''' Gets the name of the resultset column with which the attributed member 
    ''' is associated.
    ''' </summary>
    ''' <remarks>
    ''' <para>This value can be null, in which case the data server should assume the
    ''' resultset column to have the same name as the attributed member.</para>
    ''' </remarks>
    Public ReadOnly Property ColumnName() As String
        Get
            Return _ColumnName
        End Get
    End Property

    ''' <summary>
    ''' Gets / Sets the value the attributed member should receive when the associated
    ''' resultset column is in a null state.
    ''' </summary>
    ''' <remarks>
    ''' <para>This value may itself be null, in which case the data server should not 
    ''' set the attributed member, allowing it to retain its default value.</para>
    ''' </remarks>
    Public Property NullValue() As Object
        Get
            Return _NullValue
        End Get
        Set(ByVal value As Object)
            _NullValue = value
        End Set
    End Property

End Class

