
    ''' <summary>
    ''' This attribute should be applied to the properties and fields of 
    ''' call objects, to indicate that the attributed member is a database
    ''' command parameter.
    ''' </summary>
    ''' <remarks>All properties of this attribute are optional.</remarks>
    <AttributeUsage(AttributeTargets.Field Or AttributeTargets.Property, AllowMultiple:=False, Inherited:=True)> _
    Public Class ParameterAttribute
        Inherits DbAttribute

        Private _Direction As ParameterDirection
        Private _Name As String
        Private _Size As Int32
        Private _Type As DbType

        ''' <summary>
        ''' Constructs a new instance of the attribute, with default property values.
        ''' </summary>
        Public Sub New()
            _Direction = ParameterDirection.Input
            _Size = -1
            _Type = CType(-1, DbType)
        End Sub

        ''' <summary>
        ''' Gets / Sets the <see cref="ParameterDirection"/> of the parameter 
        ''' described by this attribute.
        ''' </summary>
        ''' <remarks>
        ''' <para>This property is optional, and defaults to 
        ''' <see cref="ParameterDirection.Input" />.</para>
        ''' </remarks>
        Public Property Direction() As ParameterDirection
            Get
                Return _Direction
            End Get
            Set(ByVal value As ParameterDirection)
                If value = ParameterDirection.ReturnValue Then Throw New ArgumentException("ParameterDirection.ReturnValue is not allowed here.")
                _Direction = value
            End Set
        End Property

        ''' <summary>
        ''' Gets / Sets the size of the parameter described by this attribute, in
        ''' bytes.
        ''' </summary>
        ''' <remarks>
        ''' <para>This property is, in most cases, optional and defaults to -1, 
        ''' which indicates that the ADO.NET provider should infer the parameter 
        ''' size from its data type.</para>
        ''' <para>This property is required if <see cref="ParameterDirection" /> is
        ''' <see cref="ParameterDirection.Output" /> or 
        ''' <see cref="ParameterDirection.InputOutput" /> and 
        ''' <see cref="Type" /> is <see cref="DbType.Binary" /> or
        ''' <see cref="DbType.[String]" />"
        ''' </para>
        ''' </remarks>
        Public Property Size() As Int32
            Get
                Return _Size
            End Get
            Set(ByVal value As Int32)
                _Size = value
            End Set
        End Property

        ''' <summary>
        ''' Gets / Sets the name of the parameter described by this attribute.
        ''' </summary>
        ''' <remarks>
        ''' <para>This property is optional. If null, the parameter name defaults
        ''' to the name of the attributed member.</para>
        ''' </remarks>
        Public Property Name() As String
            Get
                Return _Name
            End Get
            Set(ByVal value As String)
                _Name = value
            End Set
        End Property

        ''' <summary>
        ''' Gets / Sets the data type of the parameter described by this attribute.
        ''' </summary>
        ''' <remarks>
        ''' <para>This property is optional. If not specified, the data type of the
        ''' parameter described by this attribute should be inferred from its value.</para>
        ''' </remarks>
        Public Property Type() As DbType
            Get
                Return _Type
            End Get
            Set(ByVal value As DbType)
                _Type = value
            End Set
        End Property

    End Class


